@@ -1,6 +1,6 @@
 ;;; pomm-third-time.el --- Implementation of the third time technique in Emacs -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022 Korytov Pavel
+;; Copyright (C) 2023 Korytov Pavel
 
 ;; Author: Korytov Pavel <thexcloud@gmail.com>
 ;; Maintainer: Korytov Pavel <thexcloud@gmail.com>
@@ -35,6 +35,11 @@
 (require 'transient)
 (require 'calc)
 
+;; XXX optional dependency on org-clock
+(declare-function org-clock-in-last "org-clock")
+(declare-function org-clock-out "org-clock")
+(defvar org-clock-current-task)
+
 (defgroup pomm-third-time nil
   "Third Time timer implementation."
   :group 'pomm)
@@ -90,21 +95,21 @@ of period.  The format is as follows:
   "The current state of the Third Time timer.
 
 This is an alist with the following keys:
-- status: either 'stopped or 'running
+- status: either \\='stopped or \\='running
   (having a pause state seems to make little sense here)
 - current: an alist with a current period
 - history: a list with history for today
 - last-changed-time: a timestamp of the last change in status
 - context: a string that describes the current task
 
-'current is an alist with the following keys:
-- kind: either 'work or 'break
+\\='current is an alist with the following keys:
+- kind: either \\='work or \\='break
 - start-time: start timestamp
 - break-time-bank: break time, postpone from previous periods
 - iteration: number of the current iteration
 
-'history is a list of alists with the following keys:
-- kind: same as in 'current
+\\='history is a list of alists with the following keys:
+- kind: same as in \\='current
 - iteration
 - start-time: start timestamp
 - end-time: end timestamp
@@ -126,7 +131,7 @@ This is an alist with the following keys:
           (last-changed-time ,(time-convert nil 'integer)))
         pomm-current-mode-line-string "")
   (setf (alist-get 'status pomm-third-time--state) 'stopped)
-  (run-hooks 'pomm-on-status-changed-hook))
+  (run-hooks 'pomm-third-time-on-status-changed-hook))
 
 (defun pomm-third-time--init-state ()
   "Initialize the Third Time timer state."
@@ -186,7 +191,7 @@ If the variable is nil, the function does nothing."
              (symbol-name (alist-get 'kind (alist-get 'current pomm-third-time--state)))
              (or (alist-get 'iteration (alist-get 'current pomm-third-time--state)) 0)
              (pomm-third-time--break-time)
-             (alist-get 'context pomm--state))
+             (alist-get 'context pomm-third-time--state))
      nil pomm-third-time-csv-history-file 'append 1)))
 
 (transient-define-prefix pomm-third-time-reset ()
@@ -248,14 +253,17 @@ return it, otherwise, return a value like a calc error."
         ('nil 0)))
    0))
 
-(defun pomm-third-time--worked-time ()
-  "Get total time worked in the current iteration."
-  (let ((iteration (alist-get 'iteration
-                              (alist-get 'current pomm-third-time--state)))
-        (current-kind (alist-get 'kind (alist-get 'current pomm-third-time--state))))
+(defun pomm-third-time--total-time (&optional kind)
+  "Get total time spent in `state` in the current iteration.
+
+KIND is the same as in `pomm-third-time--state'."
+  (let* ((kind (if kind kind 'work))
+         (iteration (alist-get 'iteration
+                               (alist-get 'current pomm-third-time--state)))
+         (current-kind (alist-get 'kind (alist-get 'current pomm-third-time--state))))
     (apply
      #'+
-     (if (eq current-kind 'work)
+     (if (eq current-kind kind)
          (pomm-third-time--current-period-time)
        0)
      (mapcar
@@ -265,9 +273,10 @@ return it, otherwise, return a value like a calc error."
       (seq-filter
        (lambda (item)
          (and (= (alist-get 'iteration item) iteration)
-              (eq (alist-get 'kind item) 'work)))
+              (eq (alist-get 'kind item) kind)))
        (alist-get 'history pomm-third-time--state))))))
 
+
 (defun pomm-third-time--need-switch-p ()
   "Check if the break period has to end."
   (and
@@ -311,7 +320,7 @@ KIND is the same as in `pomm-third-time--state'"
                       (format "\nBreak time remaining: %s"
                               (pomm-third-time--format-period
                                (pomm-third-time--break-time)))))))
-   :title "Pomodoro"))
+   :title "Third Time"))
 
 (defun pomm-third-time--switch ()
   "Switch between periods."
@@ -436,6 +445,23 @@ Take a look at the `pomm-third-time' function for more details."
   (setf (alist-get 'context pomm-third-time--state)
         (prin1-to-string (read-minibuffer "Context: " (current-word)))))
 
+(defun pomm-third-time--sync-org-clock ()
+  "Sync org-clock with the pomodoro timer."
+  (let* ((status (alist-get 'status pomm-third-time--state))
+         (kind (alist-get 'kind (alist-get 'current pomm-third-time--state)))
+         (active-p (and (eq kind 'work)
+                        (eq status 'running)))
+         (resume-next-time-p (not (eq status 'stopped))))
+    (cond
+     ((and active-p (not org-clock-current-task)
+           pomm--sync-org-clock-was-stopped)
+      (if pomm-org-clock-in-immediately
+          (org-clock-in-last)
+        (pomm--org-clock-in-last-after-action)))
+     ((and (not active-p) org-clock-current-task)
+      (org-clock-out)
+      (setq pomm--sync-org-clock-was-stopped resume-next-time-p)))))
+
 ;;;###autoload
 (defun pomm-third-time-start-with-context ()
   "Prompt for context call call `pomm-third-time-start'."
@@ -536,7 +562,9 @@ KIND is the same as in `pomm-third-time--state'"
             (iteration (alist-get 'iteration
                                   (alist-get 'current pomm-third-time--state)))
             (break-time (pomm-third-time--break-time))
-            (period-time (pomm-third-time--current-period-time)))
+            (period-time (pomm-third-time--current-period-time))
+            (total-work (pomm-third-time--total-time 'work))
+            (total-break (pomm-third-time--total-time 'break)))
         (concat
          (format "Iteration #%d. " iteration)
          "State: "
@@ -556,9 +584,17 @@ KIND is the same as in `pomm-third-time--state'"
          (propertize
           (pomm-third-time--format-period break-time)
           'face 'success)
-         ". Total time worked: "
+         ".\nTotal time worked: "
          (propertize
-          (pomm-third-time--format-period (pomm-third-time--worked-time))
+          (pomm-third-time--format-period total-work)
+          'face 'success)
+         ". Total time braked: "
+         (propertize
+          (pomm-third-time--format-period total-break)
+          'face 'success)
+         ". Total time tracked: "
+         (propertize
+          (pomm-third-time--format-period (+ total-work total-break))
           'face 'success))))))
 
 (defclass pomm-third-time--transient-history (transient-suffix)
@@ -649,16 +685,16 @@ The idea of the technique is as follows:
 
 The timer can have two states:
 - Stopped.
-  Can be started with 's' or `pomm-third-time-start'.
+  Can be started with \\='s' or `pomm-third-time-start'.
 - Running.
-  Can be stopped with 'S' or `pomm-third-time-stop'.
+  Can be stopped with \\='S' or `pomm-third-time-stop'.
 
 If the timer is running, the current period type (work or break) can
-be switched by 'b' or `pomm-third-time-switch'.  If the break time
+be switched by \\='b' or `pomm-third-time-switch'.  If the break time
 runs out, the timer automatically switches to work.
 
 The timer supports setting \"context\", for example, a task on which
-you're working on.  It can be set with '-c' or
+you\\='re working on.  It can be set with \\='-c' or
 `pomm-third-time-set-context'.  This is useful together with CSV
 logging, which is enabled if `pomm-third-time-csv-history-file' is
 non-nil.
