@@ -1,13 +1,14 @@
 ;;; pomm.el --- Pomodoro and Third Time timers -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022 Korytov Pavel
+;; Copyright (C) 2023 Korytov Pavel
 
 ;; Author: Korytov Pavel <thexcloud@gmail.com>
 ;; Maintainer: Korytov Pavel <thexcloud@gmail.com>
-;; Package-Version: 0.2.0
-;; Package-Revision: 0942131ac3f2
+;; Package-Version: 20251223.1344
+;; Package-Revision: f198afa7519c
 ;; Package-Requires: ((emacs "27.1") (alert "1.2") (seq "2.22") (transient "0.3.0"))
 ;; Homepage: https://github.com/SqrtMinusOne/pomm.el
+;; Published-At: 2021-11-05
 
 ;; This file is NOT part of GNU Emacs.
 
@@ -45,6 +46,11 @@
 (require 'eieio)
 (require 'transient)
 
+;; XXX optional dependency on org-clock
+(declare-function org-clock-in-last "org-clock")
+(declare-function org-clock-out "org-clock")
+(defvar org-clock-current-task)
+
 (defgroup pomm nil
   "Pomodoro and Third Time timers."
   :group 'tools)
@@ -103,7 +109,7 @@
   (locate-user-emacs-file "pomm")
   "Location of the pomm state file."
   :group 'pomm
-  :type 'string)
+  :type 'file)
 
 (defcustom pomm-history-reset-hour 0
   "An hour on which the history will be reset.
@@ -135,7 +141,7 @@ of period.  The format is as follows:
 - iteration
 - context"
   :group 'pomm
-  :type 'string)
+  :type '(choice file (const nil)))
 
 (defcustom pomm-csv-history-file-timestamp-format "%s"
   "Timestamp format in the csv file.
@@ -209,18 +215,25 @@ Each element of the list is a cons cell, where:
   :group 'pomm
   :type 'hook)
 
+(defcustom pomm-org-clock-in-immediately t
+  "Run `org-clock-in-last' immediately after the break ends.
+
+If nil, the clock in happens after the first command."
+  :group 'pomm
+  :type 'boolean)
+
 (defvar pomm--state nil
   "The current state of the Pomodoro timer.
 
 This is an alist with the following keys:
-- status: either 'stopped, 'paused or 'running
+- status: either \\='stopped, \\='paused or \\='running
 - current: an alist with a current period
 - history: a list with history for today
 - last-changed-time: a timestamp of the last change in status
 - context: a string that describes the current task
 
-'current is also an alist with the following keys:
-- kind: either 'short-break, 'long-break or 'work
+\\='current is also an alist with the following keys:
+- kind: either \\='short-break, \\='long-break or \\='work
 - start-time: start timestamp
 - effective-start-time: start timestamp, corrected for pauses
 - iteration: number of the current Pomodoro iteration
@@ -477,7 +490,7 @@ The condition is: (effective-start-time + length) < now."
 
 The formula is:
 \(effective-start-time + length\) - now + paused-time,
-where paused-time is 0 if status is not 'paused, otherwise:
+where paused-time is 0 if status is not \\='paused, otherwise:
 paused-time := now - last-changed-time"
   (+
    (+ (or (alist-get 'effective-start-time (alist-get 'current pomm--state)) 0)
@@ -536,6 +549,37 @@ minor mode."
       (remove-hook 'pomm-on-status-changed-hook #'pomm-update-mode-line-string)
       (remove-hook 'pomm-on-status-changed-hook #'force-mode-line-update))))
 
+(defvar pomm--sync-org-clock-was-stopped nil
+  "If t, `pomm--sync-org-clock' had stopped `org-clock.")
+
+(defun pomm--org-clock-in-last-after-action ()
+  "Run `org-clock-in-last' after some action by the user.
+
+This exists because I sometimes return to PC after a the break ends."
+  (add-hook 'post-command-hook #'pomm--org-clock-in-last-and-remove-from-hook))
+
+(defun pomm--org-clock-in-last-and-remove-from-hook ()
+  "Run `org-clock-in-last' and remove self from the `post-command-hook'."
+  (org-clock-in-last)
+  (remove-hook 'post-command-hook #'pomm--org-clock-in-last-and-remove-from-hook))
+
+(defun pomm--sync-org-clock ()
+  "Sync org-clock with the pomodoro timer."
+  (let* ((status (alist-get 'status pomm--state))
+         (kind (alist-get 'kind (alist-get 'current pomm--state)))
+         (active-p (and (eq kind 'work)
+                        (eq status 'running)))
+         (resume-next-time-p (not (eq status 'stopped))))
+    (cond
+     ((and active-p (not org-clock-current-task)
+           pomm--sync-org-clock-was-stopped)
+      (if pomm-org-clock-in-immediately
+          (org-clock-in-last)
+        (pomm--org-clock-in-last-after-action)))
+     ((and (not active-p) org-clock-current-task)
+      (org-clock-out)
+      (setq pomm--sync-org-clock-was-stopped resume-next-time-p)))))
+
 ;;;###autoload
 (defun pomm-start ()
   "Start or continue the pomodoro timer.
@@ -828,17 +872,17 @@ This command initializes the timer and triggers the transient buffer.
 
 The timer can have 3 states:
 - Stopped.
-  Can be started with 's' or `pomm-start'.  A new iteration of the
+  Can be started with \\='s' or `pomm-start'.  A new iteration of the
   timer will be started.
 - Paused.
-  Can be continuted with 's' / `pomm-start' or stopped competely with
-  'S' / `pomm-stop'.
+  Can be continuted with \\='s' / `pomm-start' or stopped competely with
+  \\='S' / `pomm-stop'.
 - Running.
-  Can be paused with 'p' / `pomm-pause' or stopped with 'S' /
+  Can be paused with \\='p' / `pomm-pause' or stopped with \\='S' /
   `pomm-stop'.
 
 The timer supports setting \"context\", for example, a task on which
-you're working on.  It can be set with '-c' or `pomm-set-context'.
+you're working on.  It can be set with \\='-c' or `pomm-set-context'.
 This is useful together with CSV logging, which is enabled if
 `pomm-csv-history-file' is non-nil.
 
